import React, {useContext} from "react";
import {Smartable, useTable} from "./Smartable";

/**
 * Basic column key type.
 */
export type ColumnKey = string|symbol;

/**
 * Smartable columns definition.
 */
export type Columns<K extends ColumnKey> = Record<K, Column>;

/**
 * Define columns for a Smartable.
 * @param columns
 */
export function createColumns<K extends ColumnKey>(...columns: [K, Column][]): Columns<K>
{
	return Object.fromEntries(
		columns,
	) as Columns<K>;
}

/**
 * Smartable column definition.
 */
export interface Column
{
	/**
	 * Column title element.
	 */
	title: React.ReactNode;

	/**
	 * Column cell default element.
	 */
	cellElement?: React.ReactElement;
}

/**
 * Create a column for a Smartable.
 * @param key The column key, which should be unique per Smartable instance.
 * @param column The column definition.
 */
export function createColumn<K extends ColumnKey>(key: K, column: Column): [K, Column]
{
	return [key, column];
}

/**
 * Table column context data.
 */
export interface ColumnContextData<CK extends ColumnKey>
{
	/**
	 * Column key.
	 */
	key: ColumnKey;

	/**
	 * Column definition.
	 */
	column: Column;
}

export const ColumnContext = React.createContext<ColumnContextData<ColumnKey>>(undefined);

/**
 * Hook to get current column data.
 */
export function useColumn<CK extends ColumnKey>(smartable?: Smartable<CK>): ColumnContextData<CK>
{
	return useContext(ColumnContext);
}

/**
 * Default column heading component.
 */
export function ColumnHeading()
{
	// Get current column data.
	const {column} = useColumn();
	return <th>{column.title}</th>;
}

/**
 * Auto column context provider from table context.
 */
export function AutoColumnContextProvider({columnKey, children}: React.PropsWithChildren<{columnKey: ColumnKey;}>)
{
	// Get table data.
	const table = useTable();

	return (
		<ColumnContext.Provider value={{
			key: columnKey,
			// Get current column data from table data.
			column: table.columns[columnKey],
		}}>
			{children}
		</ColumnContext.Provider>
	)
}
